# -*- coding: utf-8 -*-
"""
serializer handlers exceptions module.
"""

from pyrin.core.exceptions import CoreException


class SerializerHandlersException(CoreException):
    """
    serializer handlers exception.
    """
    pass
