# -*- coding: utf-8 -*-
"""
city package.
"""

from pyrin.packaging.base import Package


class CityPackage(Package):
    """
    city package class.
    """

    NAME = __name__
    DEPENDS = ['demo.common.province']
    COMPONENT_NAME = 'common.city.component'
