# -*- coding: utf-8 -*-
"""
province package.
"""

from pyrin.packaging.base import Package


class ProvincePackage(Package):
    """
    province package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'common.province.component'
