# -*- coding: utf-8 -*-
"""
security package.
"""

from pyrin.security import SecurityPackage as BaseSecurityPackage


class SecurityPackage(BaseSecurityPackage):
    """
    security package class.
    """

    NAME = __name__
