# -*- coding: utf-8 -*-
"""
response wrappers package.
"""

from pyrin.packaging.base import Package


class ResponseWrappersPackage(Package):
    """
    response wrappers package class.
    """

    NAME = __name__
