# -*- coding: utf-8 -*-
"""
template enumerations module.
"""

from pyrin.core.enumerations import CoreEnum


class TemplateCLIHandlersEnum(CoreEnum):
    """
    template cli handlers enum.
    """

    PACKAGE = 'package'
    EMPTY_PACKAGE = 'empty_package'
