# -*- coding: utf-8 -*-
"""
users package.
"""

from pyrin.security.users import UsersPackage as BaseUsersPackage


class UsersPackage(BaseUsersPackage):
    """
    users package class.
    """

    NAME = __name__
