# -*- coding: utf-8 -*-
"""
validator package.
"""

from pyrin.packaging.base import Package


class ValidatorPackage(Package):
    """
    validator package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'validator.component'
