# -*- coding: utf-8 -*-
"""
authentication package.
"""

from pyrin.security.authentication import AuthenticationPackage as BaseAuthenticationPackage


class AuthenticationPackage(BaseAuthenticationPackage):
    """
    authentication package class.
    """

    NAME = __name__
