# -*- coding: utf-8 -*-
"""
demo application main package.
"""

from pyrin.application.base import Application


__version__ = '0.2.2'


class DemoApplication(Application):
    """
    demo application class.

    server should create an instance of this class on startup.
    """
    pass
