# -*- coding: utf-8 -*-
"""
accounting package.
"""

from pyrin.packaging.base import Package


class AccountingPackage(Package):
    """
    accounting package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'accounting.component'
