# -*- coding: utf-8 -*-
"""
router handlers package.
"""

from pyrin.packaging.base import Package


class RouterHandlersPackage(Package):
    """
    router handlers package class.
    """

    NAME = __name__
    DEPENDS = []
