# -*- coding: utf-8 -*-
"""
sentry integrations package.
"""

from pyrin.packaging.base import Package


class SentryIntegrationsPackage(Package):
    """
    sentry integrations package class.
    """

    NAME = __name__
