# -*- coding: utf-8 -*-
"""
caching remote package.
"""

from pyrin.packaging.base import Package


class CachingRemotePackage(Package):
    """
    caching remote package class.
    """

    NAME = __name__
